"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestRequest = void 0;
var tslib_1 = require("tslib");
var rxjs_1 = require("rxjs");
var rxjs_2 = require("rxjs");
var tnp_core_1 = require("tnp-core");
var models_1 = require("./models");
var rest_headers_1 = require("./rest-headers");
var tnp_core_2 = require("tnp-core");
var ng2_logger_1 = require("ng2-logger");
var axios_1 = require("axios");
var resource_service_1 = require("./resource-service");
var ng2_logger_2 = require("ng2-logger");
var request_cache_1 = require("./request-cache");
var log = ng2_logger_2.Log.create('[ng2-rest] rest-request', ng2_logger_1.Level.__NOTHING);
var jobIDkey = 'jobID';
var customObs = 'customObs';
var cancelFn = 'cancelFn';
var isCanceled = 'isCanceled';
//#region mock request
//#endregion
var RestRequest = exports.RestRequest = /** @class */ (function () {
    function RestRequest() {
        this.subjectInuUse = {};
        this.meta = {};
        //#endregion
        this.replaySubjects = {};
    }
    RestRequest.prototype.handlerResult = function (options, sourceRequest) {
        if (tnp_core_1._.isUndefined(options)) {
            options = {};
        }
        // log.d(`HANDLE RESULT (jobid:${options.jobid}) ${sourceRequest.url}`);
        var res = options.res, jobid = options.jobid, isArray = options.isArray, method = options.method;
        if (typeof res !== 'object') {
            throw new Error('No resposnse for request. ');
        }
        if (tnp_core_2.Helpers.isBrowser) {
            res.headers = rest_headers_1.RestHeaders.from(res.headers);
        }
        // error no internet
        if (res.error) {
            this.subjectInuUse[jobid].error(new models_1.Models.HttpResponseError(res.error, res.data, res.headers, res.code, jobid));
            return;
        }
        var entity = this.meta[jobid].entity;
        var circular = this.meta[jobid].circular;
        var success = resource_service_1.Resource['_listenSuccess'];
        var reqResp = new models_1.Models.HttpResponse(sourceRequest, res.data, res.headers, res.code, entity, circular, jobid, isArray);
        success.next(reqResp);
        this.subjectInuUse[jobid].next(reqResp);
        this.meta[jobid] = void 0;
        this.subjectInuUse[jobid].complete();
    };
    RestRequest.prototype.checkCache = function (sourceRequest, jobid) {
        var existedInCache = request_cache_1.RequestCache.findBy(sourceRequest);
        if (existedInCache) {
            // log.i('cache exists', existedInCache)
            var success = resource_service_1.Resource['_listenSuccess'];
            success.next(existedInCache.response);
            this.subjectInuUse[jobid].next(existedInCache);
            this.subjectInuUse[jobid].complete();
            return true;
        }
        // log.i(`cache not exists for jobid ${jobid}`)
        return false;
    };
    RestRequest.prototype.req = function (url, method, headers, body, jobid, isArray, mockHttp) {
        if (isArray === void 0) { isArray = false; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var CancelToken, source, response, r, headersJson, responseType, catchedError_1, err, msg, stack, errObs, error;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.checkCache({
                            url: url,
                            body: body,
                            isArray: isArray,
                            method: method
                        }, jobid)) {
                            return [2 /*return*/];
                        }
                        CancelToken = axios_1.default.CancelToken;
                        source = CancelToken.source();
                        this.subjectInuUse[jobid][cancelFn] = source.cancel;
                        if (mockHttp) {
                            if (typeof mockHttp === 'object') {
                                response = {
                                    data: mockHttp.data,
                                    status: mockHttp.code,
                                    headers: mockHttp.headers,
                                    statusText: mockHttp.error,
                                    config: {}
                                };
                            }
                            else if (typeof mockHttp === 'function') {
                                r = mockHttp(url, method, headers, body);
                                response = {
                                    data: r.data,
                                    status: r.code,
                                    headers: r.headers,
                                    statusText: r.error,
                                    config: {}
                                };
                            }
                        }
                        headersJson = headers.toJSON();
                        responseType = headersJson.responsetypeaxios ? headersJson.responsetypeaxios : 'text';
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        if (!!response) return [3 /*break*/, 3];
                        return [4 /*yield*/, (0, axios_1.default)({
                                url: url,
                                method: method,
                                data: body,
                                responseType: responseType,
                                headers: headersJson,
                                cancelToken: source.token,
                                // withCredentials: true, // this can be done manually
                            })];
                    case 2:
                        // console.log(`[${method}] (jobid=${jobid}) request to:  ${url}`);
                        // console.log('headers axios:', headers.toJSON())
                        // console.log({ responseType, headersJson, body, method, url })
                        response = _a.sent();
                        _a.label = 3;
                    case 3:
                        // console.log('AXIOS RESPONES', response)
                        if (this.subjectInuUse[jobid][isCanceled]) {
                            return [2 /*return*/];
                        }
                        this.handlerResult({
                            res: {
                                code: response.status,
                                data: response.data,
                                isArray: isArray,
                                jobid: jobid,
                                headers: rest_headers_1.RestHeaders.from(response.headers)
                            },
                            method: method,
                            jobid: jobid,
                            isArray: isArray
                        }, {
                            url: url,
                            body: body,
                            method: method,
                            isArray: isArray,
                        });
                        return [3 /*break*/, 5];
                    case 4:
                        catchedError_1 = _a.sent();
                        if (this.subjectInuUse[jobid][isCanceled]) {
                            return [2 /*return*/];
                        }
                        // console.log('ERROR RESPONESE catchedError typeof ', typeof catchedError)
                        // console.log('ERROR RESPONESE catchedError', catchedError)
                        if (typeof catchedError_1 === 'object' && catchedError_1.response && catchedError_1.response.data) {
                            err = catchedError_1.response.data;
                            msg = catchedError_1.response.data.message || '';
                            stack = (err.stack || '').split('\n');
                            errObs = resource_service_1.Resource['_listenErrors'];
                            errObs.next({
                                msg: msg,
                                stack: stack,
                                data: catchedError_1.response.data
                            });
                        }
                        error = (catchedError_1 && catchedError_1.response) ? "[".concat(catchedError_1.response.statusText, "]: ") : '';
                        this.handlerResult({
                            res: {
                                code: (catchedError_1 && catchedError_1.response) ? catchedError_1.response.status : void 0,
                                error: "".concat(error).concat(catchedError_1.message),
                                data: (catchedError_1 && catchedError_1.response) ? JSON.stringify(catchedError_1.response.data) : void 0,
                                isArray: isArray,
                                jobid: jobid,
                                headers: rest_headers_1.RestHeaders.from(catchedError_1 && catchedError_1.response && catchedError_1.response.headers)
                            },
                            method: method,
                            jobid: jobid,
                            isArray: isArray
                        }, {
                            url: url,
                            body: body,
                            isArray: isArray,
                            method: method
                        });
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    RestRequest.prototype.getReplay = function (method, meta, onlyGetLastReplayForMethod) {
        var _this = this;
        var replay;
        //#region prevent empty tree
        if (tnp_core_1._.isUndefined(this.replaySubjects[meta.endpoint])) {
            // log.i(`(${meta.endpoint}) `);
            this.replaySubjects[meta.endpoint] = {};
        }
        if (tnp_core_1._.isUndefined(this.replaySubjects[meta.endpoint][meta.path])) {
            // log.i(`(${meta.endpoint})(${meta.path}) `);
            this.replaySubjects[meta.endpoint][meta.path] = {};
        }
        if (tnp_core_1._.isUndefined(this.replaySubjects[meta.endpoint][meta.path][method])) {
            // log.i(`(${meta.endpoint})(${meta.path}) `);
            this.replaySubjects[meta.endpoint][meta.path][method] = {};
        }
        //#endregion
        var objectIDToCreateOrLast = (Object.keys(this.replaySubjects[meta.endpoint][meta.path][method]).length) +
            (onlyGetLastReplayForMethod ? 0 : 1);
        if (onlyGetLastReplayForMethod && (objectIDToCreateOrLast === 0)) {
            return replay;
        }
        if (tnp_core_1._.isUndefined(this.replaySubjects[meta.endpoint][meta.path][method][objectIDToCreateOrLast])) {
            // log.i(`(${meta.endpoint})(${meta.path})(${method}) `);
            this.replaySubjects[meta.endpoint][meta.path][method][objectIDToCreateOrLast] = {
                subject: new rxjs_2.Subject(),
                data: void 0,
            };
        }
        replay = this.replaySubjects[meta.endpoint][meta.path][method][objectIDToCreateOrLast];
        if (!tnp_core_1._.isNumber(replay.id)) {
            if (RestRequest.jobId === Number.MAX_SAFE_INTEGER) {
                RestRequest.jobId = 0;
            }
            var jobid_1 = RestRequest.jobId++;
            replay.id = jobid_1;
            var subject_1 = replay.subject;
            subject_1[jobIDkey] = jobid_1; // modify internal rxjs subject obj
            this.meta[jobid_1] = meta;
            this.subjectInuUse[jobid_1] = subject_1;
            this.subjectInuUse[jobid_1][customObs] = new rxjs_1.Observable(function (observer) {
                // observer.remove(() => {
                // });
                observer.add(function () {
                    // console.log(`cancel observable job${jobid}`)
                    if (!_this.subjectInuUse[jobid_1][isCanceled]) {
                        _this.subjectInuUse[jobid_1][isCanceled] = true;
                        if (typeof _this.subjectInuUse[jobid_1][cancelFn] === 'function') {
                            _this.subjectInuUse[jobid_1][cancelFn]('[ng2-rest] on purpose canceled http request');
                        }
                    }
                    else {
                        // console.log(`somehow second time cancel ${jobid}`)
                    }
                });
                var sub = subject_1.subscribe({
                    next: function (a) { return observer.next(a); },
                    error: function (a) { return observer.error(a); },
                    complete: function () {
                        sub.unsubscribe();
                        observer.complete();
                    },
                });
            });
            //#region DISPOSE  @UNCOMMENT AFTER TESTS
            // if (objectIDToCreateOrLast > 2) {
            //   const oldReq: Models.ReplayData = this.replaySubjects[meta.endpoint][meta.path][method][(objectIDToCreateOrLast - 2)];
            //   if (_.isUndefined(this.meta[oldReq.id])) {
            //     // cant delete this - for counter purpose
            //     this.replaySubjects[meta.endpoint][meta.path][method][(objectIDToCreateOrLast - 2)] = {};
            //     delete this.subjectInuUse[oldReq.id];
            //     delete this.meta[oldReq.id];
            //   }
            // }
            //#endregion
        }
        return replay;
    };
    //#region http methods
    RestRequest.prototype.generalReq = function (method, url, body, headers, meta, isArray, mockHttp) {
        var replay = this.getReplay(method, meta, false);
        replay.data = { url: url, body: body, headers: headers, isArray: isArray };
        (function (pthis, purl, pmethod, pheaders, pbody, pid, pisArray, pmockHttp) {
            // log.d(`for ${purl} jobid ${pid}`);
            setTimeout(function () { return pthis.req(purl, pmethod, pheaders, pbody, pid, pisArray, pmockHttp); });
        })(this, url, method, headers, body, replay.id, isArray, mockHttp);
        var resp = (0, rxjs_1.firstValueFrom)(replay.subject[customObs]);
        resp.observable = replay.subject[customObs];
        resp.cache = request_cache_1.RequestCache.findBy({
            body: body,
            isArray: isArray,
            method: method,
            url: url
        });
        return resp;
    };
    RestRequest.prototype.get = function (url, body, headers, meta, isArray, mockHttp) {
        return this.generalReq('get', url, body, headers, meta, isArray, mockHttp);
    };
    RestRequest.prototype.head = function (url, body, headers, meta, isArray, mockHttp) {
        return this.generalReq('head', url, body, headers, meta, isArray, mockHttp);
    };
    RestRequest.prototype.delete = function (url, body, headers, meta, isArray, mockHttp) {
        return this.generalReq('delete', url, body, headers, meta, isArray, mockHttp);
    };
    RestRequest.prototype.post = function (url, body, headers, meta, isArray, mockHttp) {
        return this.generalReq('post', url, body, headers, meta, isArray, mockHttp);
    };
    RestRequest.prototype.put = function (url, body, headers, meta, isArray, mockHttp) {
        return this.generalReq('put', url, body, headers, meta, isArray, mockHttp);
    };
    RestRequest.prototype.patch = function (url, body, headers, meta, isArray, mockHttp) {
        return this.generalReq('patch', url, body, headers, meta, isArray, mockHttp);
    };
    RestRequest.prototype.jsonp = function (url, body, headers, meta, isArray, mockHttp) {
        var _this = this;
        var replay = this.getReplay('jsonp', meta, false);
        var jobid = replay.id;
        var method = 'jsonp';
        setTimeout(function () {
            if (url.endsWith('/'))
                url = url.slice(0, url.length - 1);
            var num = Math.round(10000 * Math.random());
            var callbackMethodName = "cb_" + num;
            window[callbackMethodName] = function (data) {
                if (_this.checkCache({
                    url: url,
                    body: body,
                    isArray: isArray,
                    method: method
                }, jobid)) {
                    return;
                }
                _this.handlerResult({
                    res: {
                        data: data,
                        isArray: isArray
                    },
                    method: method,
                    jobid: jobid,
                    isArray: isArray
                }, {
                    url: url,
                    body: body,
                    isArray: isArray,
                    method: method,
                });
            };
            var sc = document.createElement('script');
            sc.src = "".concat(url, "?callback=").concat(callbackMethodName);
            document.body.appendChild(sc);
            document.body.removeChild(sc);
        });
        var resp = (0, rxjs_1.firstValueFrom)(replay.subject[customObs]);
        resp.observable = replay.subject[customObs];
        console.log('assiging custom observable');
        resp.cache = request_cache_1.RequestCache.findBy({
            body: body,
            isArray: isArray,
            method: method,
            url: url
        });
        return resp;
    };
    RestRequest.prototype.replay = function (method, meta) {
        var _this = this;
        var replay = this.getReplay(method, meta, true);
        if (!replay || !replay.data) {
            console.warn("Canno replay first ".concat(method, " request from ").concat(meta.endpoint, "/").concat(meta.path));
            return;
        }
        ;
        if (replay && replay.subject && Array.isArray(replay.subject.observers) &&
            replay.subject.observers.length === 0) {
            console.warn("No observators for ".concat(method, " request from ").concat(meta.endpoint, "/").concat(meta.path));
            return;
        }
        var url = replay.data.url;
        var headers = replay.data.headers;
        var body = replay.data.body;
        var isArray = replay.data.isArray;
        setTimeout(function () { return _this.req(url, method, headers, body, replay.id, isArray); });
    };
    RestRequest.jobId = 0;
    return RestRequest;
}());
//# sourceMappingURL=rest-request.js.map